#!/usr/bin/perl
#
# script to get readings from mbmon and dump into rrd
#
# $Id: mbmongraph.pl 204 2007-06-23 13:54:05Z weizen_42 $
#
# created 4 July 2006 by weizen_42 at ipcop-forum dot de
#
# 2006-07-10  v0.0.2    more config options for graphs
# 2006-08-05  v0.1.1    bugfixed voltage graph. Showed Temp in title
# 2006-08-09  v0.1.3    bugfixed fan graph. Showed Temp in title
# 2006-12-17  v0.3.0    some more language hacks
# 2007-06-23  v0.4.0    display min values in graph
#

use strict;
use RRDs;
require "/var/ipcop/general-functions.pl";
require "${General::swroot}/lang.pl";

# path for rrd database
my $path_rrd = "/var/log/rrd";
# path for graphs
my $path_graphs = "/home/httpd/html/graphs";

my $temp;
my $ERROR;
my $dbg = 0;

my %mbmon_settings = ();
&General::readhash("${General::swroot}/mbmongraph/settings", \%mbmon_settings);

my %colors = ();
$colors{"1"} = "#0000FF";
$colors{"2"} = "#00FF00";
$colors{"3"} = "#FF0000";
$colors{"4"} = "#FFD700";
$colors{"5"} = "#CCCCCC";
$colors{"6"} = "#40E0D0";
$colors{"7"} = "#90EE90";
$colors{"8"} = "#F4A460";

# Added for conversion of utf-8 characters
use Encode 'from_to';
my %tr=();

# From makegraphs:
# Force language back to English (ugly hack!)
# Modified to only force if we are unable to convert charset 
# from utf-8
if ((${Lang::language} eq 'bg') || 
    (${Lang::language} eq 'el') || 
    (${Lang::language} eq 'fa') ||
    (${Lang::language} eq 'gu') ||
    (${Lang::language} eq 'ja') ||
    (${Lang::language} eq 'ru') ||
    (${Lang::language} eq 'th') || 
    (${Lang::language} eq 'ur') || 
    (${Lang::language} eq 'vi') ||
    (${Lang::language} eq 'zh') ||
    (${Lang::language} eq 'zt')) 
{
  eval `/bin/cat "${General::swroot}/langs/en.pl"`;

  # add our AddOn texts as well
  $tr{'mbmon temp in'} = 'temperature in';
  $tr{'mbmon fan in'} = 'speed in';
}
else 
{
  %tr=%Lang::tr;    # use translated version for other languages
}

my %mbmon_values = ();
my $key;
my $value;
my @args = ();
my $count = 0;

open(MBMON_OUT, ">/var/log/mbmon-values");
open(FD, "/usr/local/bin/mbmon -rc1|" ) || die "ERROR: Cannot run mbmon\n" ;

while( $_ = <FD> ) 
{
  next unless( /^([A-Za-z][^:\s]+)\s*:\s*([+\-]{0,1}[\d\.]+)/ ) ;
  $key = $1 ;
  $value = $2 ;
  $key =~ y/A-Z/a-z/ ;
  $mbmon_values{$key} = $value;
  print(MBMON_OUT "$key=$value\n");
}
close(FD);
close(MBMON_OUT);


sub updatembmondata 
{
  if ( ! -e "$path_rrd/mbmon.rrd" )
  {
    # database did not exist -> create

    @args = ("$path_rrd/mbmon.rrd");

    push(@args, "--step=300");
    foreach $key ( sort(keys %mbmon_values) ) 
    {
      push(@args, "DS:$key:GAUGE:600:U:U");
    }
    push(@args, "RRA:AVERAGE:0.5:1:576");
    push(@args, "RRA:AVERAGE:0.5:6:672");
    push(@args, "RRA:AVERAGE:0.5:24:732");
    push(@args, "RRA:AVERAGE:0.5:144:1460");

    print("create ". join( ", ", @args)) if ( $dbg );
   
    RRDs::create (@args);
    $ERROR = RRDs::error;
    print("Error in RRD::create for mbmon: $ERROR\n") if $ERROR;
  }

  my @ds;
  my @val;
  my $template;

  foreach $key ( sort(keys %mbmon_values) )
  {
    push(@ds, $key);
    push(@val, $mbmon_values{$key});
  }

  $template = join(':', @ds);
  $value    = "N:".join(':', @val);

  print("update template = '$template'\n") if ( $dbg );
  print("update value    = '$value'\n") if ( $dbg );

  RRDs::update("$path_rrd/mbmon.rrd", "-t", $template, $value);
  $ERROR = RRDs::error;
  print("Error in RRD::update for mbmon: $ERROR\n") if $ERROR;
}


sub updatetempgraph
{
  my $type   = "temp";
  my $period = $_[0];

  @args = ("$path_graphs/mbmon-$type-$period.png", "--start", "-1$period", "-aPNG", "-i", "-z",
    "--alt-y-grid", "-w 600", "-h 100", "--alt-autoscale",
    "--color", "SHADEA#EAE9EE",
    "--color", "SHADEB#EAE9EE",
    "--color", "BACK#EAE9EE",
    "-t $tr{'mbmon temp'} ($tr{'graph per'} $tr{$period})" );

  $count = 1;
  foreach $key ( sort(keys %mbmon_values) ) 
  {
    if ( (index($key, $type) != -1) && ($mbmon_settings{'LINE-'.$key} eq 'on') )
    {
      if ( !defined($mbmon_settings{'LABEL-'.$key}) || ($mbmon_settings{'LABEL-'.$key} eq '') )
      {
        $mbmon_settings{'LABEL-'.$key} = $key;
      }

      push(@args, "DEF:$key=$path_rrd/mbmon.rrd:$key:AVERAGE");
      push(@args, "LINE2:$key$colors{$count}:$mbmon_settings{'LABEL-'.$key} $tr{'mbmon temp in'} C\\j");
      push(@args, "GPRINT:$key:MIN:$tr{'mbmon minimal'}\\:%5.1lf C");
      push(@args, "GPRINT:$key:MAX:$tr{'maximal'}\\:%5.1lf C");
      push(@args, "GPRINT:$key:AVERAGE:$tr{'average'}\\:%5.1lf C");
      push(@args, "GPRINT:$key:LAST:$tr{'current'}\\:%5.1lf C\\j");

      $count++;
    }
  }

  if ( $count > 1 )
  {
    RRDs::graph ( @args );
    $ERROR = RRDs::error;
    print("Error in RRD::graph for temp: $ERROR\n")if $ERROR;
  }
}

sub updatefangraph
{
  my $type   = "fan";
  my $period = $_[0];

  @args = ("$path_graphs/mbmon-$type-$period.png", "--start", "-1$period", "-aPNG", "-i", "-z",
    "--alt-y-grid", "-w 600", "-h 100", "--alt-autoscale",
    "--color", "SHADEA#EAE9EE",
    "--color", "SHADEB#EAE9EE",
    "--color", "BACK#EAE9EE",
    "-t $tr{'mbmon fan'} ($tr{'graph per'} $tr{$period})" );

  $count = 1;
  foreach $key ( sort(keys %mbmon_values) ) 
  {
    if ( (index($key, $type) != -1) && ($mbmon_settings{'LINE-'.$key} eq 'on') )
    {
      if ( !defined($mbmon_settings{'LABEL-'.$key}) || ($mbmon_settings{'LABEL-'.$key} eq '') )
      {
        $mbmon_settings{'LABEL-'.$key} = $key;
      }

      push(@args, "DEF:$key=$path_rrd/mbmon.rrd:$key:AVERAGE");
      push(@args, "LINE2:$key$colors{$count}:$mbmon_settings{'LABEL-'.$key} $tr{'mbmon fan in'} rpm\\j");
      push(@args, "GPRINT:$key:MIN:$tr{'mbmon minimal'}\\:%5.0lf rpm");
      push(@args, "GPRINT:$key:MAX:$tr{'maximal'}\\:%5.0lf rpm");
      push(@args, "GPRINT:$key:AVERAGE:$tr{'average'}\\:%5.0lf rpm");
      push(@args, "GPRINT:$key:LAST:$tr{'current'}\\:%5.0lf rpm\\j");

      $count++;
    }
  }

  if ( $count > 1 )
  {
    RRDs::graph ( @args );
    $ERROR = RRDs::error;
    print("Error in RRD::graph for temp: $ERROR\n")if $ERROR;
  }
}

sub updatevoltgraph
{
  my $type   = "volt";
  my $period = $_[0];

  @args = ("$path_graphs/mbmon-$type-$period.png", "--start", "-1$period", "-aPNG", "-i", "-z",
    "--alt-y-grid", "-w 600", "-h 100", "--alt-autoscale",
    "--color", "SHADEA#EAE9EE",
    "--color", "SHADEB#EAE9EE",
    "--color", "BACK#EAE9EE",
    "-t $tr{'mbmon volt'} ($tr{'graph per'} $tr{$period})" );

  $count = 1;
  foreach $key ( sort(keys %mbmon_values) ) 
  {
    my $v = substr($key,0,1);
    if ( ($v eq 'v') && ($mbmon_settings{'LINE-'.$key} eq 'on') )
    {
      if ( !defined($mbmon_settings{'LABEL-'.$key}) || ($mbmon_settings{'LABEL-'.$key} eq '') )
      {
        $mbmon_settings{'LABEL-'.$key} = $key;
      }

      push(@args, "DEF:$key=$path_rrd/mbmon.rrd:$key:AVERAGE");
      push(@args, "LINE2:$key$colors{$count}:$mbmon_settings{'LABEL-'.$key} V\\j");
      push(@args, "GPRINT:$key:MIN:$tr{'mbmon minimal'}\\:%5.2lf V");
      push(@args, "GPRINT:$key:MAX:$tr{'maximal'}\\:%5.2lf V");
      push(@args, "GPRINT:$key:AVERAGE:$tr{'average'}\\:%5.2lf V");
      push(@args, "GPRINT:$key:LAST:$tr{'current'}\\:%5.2lf V\\j");

      $count++;
    }
  }

  if ( $count > 1 )
  {
    RRDs::graph ( @args );
    $ERROR = RRDs::error;
    print("Error in RRD::graph for temp: $ERROR\n")if $ERROR;
  }
}

# From makegraphs:
###
### utf8 conversion
###
if ((${Lang::language} eq 'cs') || (${Lang::language} eq 'hu') || (${Lang::language} eq 'pl') || (${Lang::language} eq 'sk')) 
{
  # Czech, Hungarian, Polish and Slovak character set
  foreach my $key(keys %Lang::tr) 
  {
    from_to($tr{$key}, "utf-8", "iso-8859-2");
  }
}
elsif (${Lang::language} eq 'lt') 
{
  # Lithuanian
  foreach my $key(keys %Lang::tr) 
  {
    from_to($tr{$key}, "utf-8", "iso-8859-4");
  }
} 
elsif (${Lang::language} eq 'tr') 
{
  # Turkish
  foreach my $key(keys %Lang::tr) 
  {
    from_to($tr{$key}, "utf-8", "iso-8859-9");
  }
} 
else 
{
  foreach my $key(keys %Lang::tr) 
  {
    from_to($tr{$key}, "utf-8", "iso-8859-1"); 
  }
}


updatembmondata();

if ( $mbmon_settings{'GRAPH_TEMP'} == 1 )
{
  updatetempgraph ("day");
  updatetempgraph ("week");
  updatetempgraph ("month");
  updatetempgraph ("year");
}
if ( $mbmon_settings{'GRAPH_FAN'} == 1 )
{
  updatefangraph ("day");
  updatefangraph ("week");
  updatefangraph ("month");
  updatefangraph ("year");
}
if ( $mbmon_settings{'GRAPH_VOLT'} == 1 )
{
  updatevoltgraph ("day");
  updatevoltgraph ("week");
  updatevoltgraph ("month");
  updatevoltgraph ("year");
}
